# -*- coding: utf-8 -*-
#
#
import RPi.GPIO as GPIO
import time
from struct import unpack

class hx711array(object):
    def begin(self,dout_pins,sck_pin,gain=128 ):
        """Initialize all HX711 modules
        @param dout_pins: array ouf out pins to read
        @param sck_pin: Clock pin connected to all modules
        @param gain: 128, 64 for channel 1, and 32 for channel 2
        """
        GPIO.setwarnings(False)    
        GPIO.setmode(GPIO.BOARD) #Layout nach PINS
        GPIO.setup(sck_pin, GPIO.OUT)
        for p in dout_pins:
            GPIO.setup(p,GPIO.IN)
        self.dout_pins=dout_pins
        self.sck_pin=sck_pin
        self.raw= bytearray([0]*4*len(self.dout_pins))
        self.values=[0]*len(self.dout_pins)
        self.gain_map={32: 2, 64: 3, 128: 1}
        self.set_gain(gain)
    
    def is_ready(self):
        """Checks if HX711 are ready
        """
        for p in self.dout_pins:
            if(GPIO.input(p)):
                return False
        return True
    
    
    def set_gain(self,gain):
        """Set the gain and the channel
        @param gain: 128, 64 for channel 1, and 32 for channel 2
        """
        self.gain=gain
        self.read()
        
    def __shiftIn(self,nr):
        """Internal method to shift in the nr-th byte
        @param nr: byte number (starts with zero)
        """
        for i in range(0,len(self.dout_pins)):
            self.raw[nr+i*4]=0
        
        for i in range(0,8):
            GPIO.output(self.sck_pin,1)
            time.sleep(1e-6)
            GPIO.output(self.sck_pin,0)
            for j in range(0,len(self.dout_pins)):
                self.raw[nr + j*4]|=GPIO.input(self.dout_pins[j])<<(7-i)
    
    def read(self,timeout=1):
        """Internal method to read the HX711 modules
        @param timeout: timeout in seconds
        """
        self.powerUp()
        timeout_time= time.time() + timeout
        while (not self.is_ready()):
            time.sleep(1e-5)
            if(time.time>timeout_time):
                return 0xf0000000
        
        value = 0
        self.__shiftIn(2);
        self.__shiftIn(1);
        self.__shiftIn(1);
        
        # set channel and gain for next reading
        for i in range(0,self.gain_map[self.gain]):
            GPIO.output(self.sck_pin,1)
            time.sleep(1e-6)
            GPIO.output(self.sck_pin,0)
            time.sleep(1e-6)
         
        for i in range(0,len(self.dout_pins)):
            if(self.raw[2 + i * 4] & 0x80):
                self.raw[3 + i * 4] = 0xff
            else:
                self.raw[3 + i * 4] = 0x00
            self.values[i]=unpack('<l', self.raw[i*4:(i+1)*4])[0]
            value+=self.values[i]
        return value
    
    def readAverage(self,times,timeout=1):
        sum=0
        value=0
        for i in range(0,times):
            value=self.read(timeout)
            if(value==0xf0000000):
                return value
            sum+=value
        return sum/times
        
    def readAverageSingle(self,times,timeout=1):
        values=[0]*len(self.dout_pins)
        for i in range(0,times):
            self.read(timeout)
            for j in range(0,len(self.dout_pins)):
                values[j]+=self.values[j]
        for j in range(0,len(self.dout_pins)):
            values[j]/=times
        return values
        
    def powerDown(self):
        GPIO.output(self.sck_pin,0)
        GPIO.output(self.sck_pin,1)

    
    def powerUp(self):
        GPIO.output(self.sck_pin,0)

